@@ -21,7 +21,7 @@
 
 #include "addon/addon.hpp"
 #include "addon/md5.hpp"
-#include "physfs/physfs_file_system.hpp"
+#include "physfs/util.hpp"
 #include "supertux/globals.hpp"
 #include "util/file_system.hpp"
 #include "util/gettext.hpp"
@@ -66,6 +66,15 @@ MD5 md5_from_file(const std::string& filename)
   }
 }
 
+MD5 md5_from_archive(const std::string& filename)
+{
+  if (physfsutil::is_directory(filename)) {
+    return MD5();
+  } else {
+    return md5_from_file(filename);
+  }
+}
+
 static Addon& get_addon(const AddonManager::AddonList& list, const AddonId& id,
                         bool installed)
 {
@@ -102,7 +111,7 @@ static std::vector<AddonId> get_addons(const AddonManager::AddonList& list)
 static PHYSFS_EnumerateCallbackResult add_to_dictionary_path(void *data, const char *origdir, const char *fname)
 {
     std::string full_path = FileSystem::join(origdir, fname);
-    if (PhysFSFileSystem::is_directory(full_path))
+    if (physfsutil::is_directory(full_path))
     {
         log_debug << "Adding \"" << full_path << "\" to dictionary search path" << std::endl;
         // We want translations from addons to have precedence
@@ -114,7 +123,7 @@ static PHYSFS_EnumerateCallbackResult add_to_dictionary_path(void *data, const c
 static PHYSFS_EnumerateCallbackResult remove_from_dictionary_path(void *data, const char *origdir, const char *fname)
 {
     std::string full_path = FileSystem::join(origdir, fname);
-    if (PhysFSFileSystem::is_directory(full_path))
+    if (physfsutil::is_directory(full_path))
     {
         g_dictionary_manager->remove_directory(full_path);
     }
@@ -554,12 +563,10 @@ AddonManager::is_from_old_addon(const std::string& filename) const
 bool
 AddonManager::is_addon_installed(const std::string& id) const
 {
-  for(const auto& installed_addon : get_installed_addons())
-  {
-    if(installed_addon == id)
-      return true;
-  }
-  return false;
+  return std::any_of(get_installed_addons().begin(), get_installed_addons().end(),
+    [id] (const auto& installed_addon) {
+      return installed_addon == id;
+    });
 }
 
 std::vector<std::string>
@@ -573,12 +580,20 @@ AddonManager::scan_for_archives() const
        PHYSFS_freeList);
   for (char** i = rc.get(); *i != nullptr; ++i)
   {
-    if (StringUtil::has_suffix(*i, ".zip"))
+    const std::string fullpath = FileSystem::join(m_addon_directory, *i);
+    if (physfsutil::is_directory(fullpath))
     {
-      std::string archive = FileSystem::join(m_addon_directory, *i);
-      if (PHYSFS_exists(archive.c_str()))
-      {
-        archives.push_back(archive);
+      // ignore dot files (e.g. '.git/')
+      if ((*i)[0] != '.') {
+        archives.push_back(fullpath);
+      }
+    }
+    else
+    {
+      if (StringUtil::has_suffix(StringUtil::tolower(*i), ".zip")) {
+        if (PHYSFS_exists(fullpath.c_str())) {
+          archives.push_back(fullpath);
+        }
       }
     }
   }
@@ -663,7 +678,7 @@ AddonManager::add_installed_addons()
 
   for (const auto& archive : archives)
   {
-    MD5 md5 = md5_from_file(archive);
+    MD5 md5 = md5_from_archive(archive);
     add_installed_archive(archive, md5.hex_digest());
   }
 }
