@@ -152,20 +152,24 @@ void
 BadGuy::update(float dt_sec)
 {
   if (!Sector::get().inside(m_col.m_bbox)) {
-    run_dead_script();
-    m_is_active_flag = false;
-    remove_me();
-    // This was removed due to fixing a bug. If is it needed somewhere, then I'm sorry. --Hume2
-    /**if(countMe) {
-      // get badguy name from sprite_name ignoring path and extension
-      std::string badguy = sprite_name.substr(0, sprite_name.length() - 7);
-      int path_chars = badguy.rfind("/",badguy.length());
-      badguy = badguy.substr(path_chars + 1, badguy.length() - path_chars);
-      // log warning since badguys_killed can no longer reach total_badguys
-      std::string current_level = "[" + Sector::get().get_level()->filename + "] ";
-      log_warning << current_level << "Counted badguy " << badguy << " starting at " << start_position << " has left the sector" <<std::endl;;
-    }*/
-    return;
+    auto this_portable = dynamic_cast<Portable*> (this);
+    if (!this_portable || !this_portable->is_grabbed())
+    {
+      run_dead_script();
+      m_is_active_flag = false;
+      remove_me();
+      // This was removed due to fixing a bug. If is it needed somewhere, then I'm sorry. --Hume2
+      /**if(countMe) {
+        // get badguy name from sprite_name ignoring path and extension
+        std::string badguy = sprite_name.substr(0, sprite_name.length() - 7);
+        int path_chars = badguy.rfind("/",badguy.length());
+        badguy = badguy.substr(path_chars + 1, badguy.length() - path_chars);
+        // log warning since badguys_killed can no longer reach total_badguys
+        std::string current_level = "[" + Sector::get().get_level()->filename + "] ";
+        log_warning << current_level << "Counted badguy " << badguy << " starting at " << start_position << " has left the sector" <<std::endl;;
+      }*/
+      return;
+    }
   }
   if ((m_state != STATE_INACTIVE) && is_offscreen()) {
     if (m_state == STATE_ACTIVE) deactivate();
@@ -309,7 +313,7 @@ BadGuy::collision_tile(uint32_t tile_attributes)
     m_in_water = false;
   }
 
-  if (tile_attributes & Tile::HURTS && is_hurtable()) {    
+  if (tile_attributes & Tile::HURTS && is_hurtable()) {
     if (tile_attributes & Tile::FIRE) {
       if (is_flammable()) ignite();
     }
@@ -383,6 +387,10 @@ BadGuy::collision_solid(const CollisionHit& hit)
 HitResponse
 BadGuy::collision_player(Player& player, const CollisionHit& )
 {
+  if (player.is_invincible()) {
+    kill_fall();
+    return ABORT_MOVE;
+  }
   if(player.get_grabbed_object() != nullptr)
   {
       auto badguy = dynamic_cast<BadGuy*>(player.get_grabbed_object());
@@ -395,10 +403,6 @@ BadGuy::collision_player(Player& player, const CollisionHit& )
         return ABORT_MOVE;
       }
   }
-  if (player.is_invincible()) {
-    kill_fall();
-    return ABORT_MOVE;
-  }
 
   //TODO: unfreeze timer
   if (m_frozen)
