@@ -18,7 +18,6 @@
 
 #include <limits>
 #include <physfs.h>
-#include <iostream>
 
 #include "audio/sound_manager.hpp"
 #include "control/input_manager.hpp"
@@ -39,7 +38,7 @@
 #include "object/player.hpp"
 #include "object/spawnpoint.hpp"
 #include "object/tilemap.hpp"
-#include "physfs/physfs_file_system.hpp"
+#include "physfs/util.hpp"
 #include "sprite/sprite_manager.hpp"
 #include "supertux/game_manager.hpp"
 #include "supertux/level.hpp"
@@ -95,7 +94,6 @@ Editor::Editor() :
   m_overlay_widget(),
   m_toolbox_widget(),
   m_layers_widget(),
-  m_scroller_widget(),
   m_enabled(false),
   m_bgr_surface(Surface::from_file("images/background/arctis2.png")),
   m_undo_manager(new UndoManager),
@@ -103,27 +101,23 @@ Editor::Editor() :
 {
   auto toolbox_widget = std::make_unique<EditorToolboxWidget>(*this);
   auto layers_widget = std::make_unique<EditorLayersWidget>(*this);
-  auto scroll_widget = std::make_unique<EditorScrollerWidget>(*this);
   auto overlay_widget = std::make_unique<EditorOverlayWidget>(*this);
 
   m_toolbox_widget = toolbox_widget.get();
   m_layers_widget = layers_widget.get();
-  m_scroller_widget = scroll_widget.get();
   m_overlay_widget = overlay_widget.get();
-
+  
   auto undo_button_widget = std::make_unique<ButtonWidget>(
     SpriteManager::current()->create("images/engine/editor/undo.png"),
-    Vector(0, 200), [this]{ undo(); });
+    Vector(10, 10), [this]{ undo(); });
   auto redo_button_widget = std::make_unique<ButtonWidget>(
     SpriteManager::current()->create("images/engine/editor/redo.png"),
-    Vector(0, 264), [this]{ redo(); });
+    Vector(60, 10), [this]{ redo(); });
 
-  // the order here is important due to how events are dispatched
   m_widgets.push_back(std::move(undo_button_widget));
   m_widgets.push_back(std::move(redo_button_widget));
   m_widgets.push_back(std::move(toolbox_widget));
   m_widgets.push_back(std::move(layers_widget));
-  m_widgets.push_back(std::move(scroll_widget));
   m_widgets.push_back(std::move(overlay_widget));
 }
 
@@ -254,7 +248,7 @@ Editor::test_level()
   std::unique_ptr<World> owned_world;
   World* current_world = m_world.get();
   if (!current_world) {
-    owned_world = World::load(directory);
+    owned_world = World::from_directory(directory);
     current_world = owned_world.get();
   }
 
@@ -331,24 +325,24 @@ Editor::update_keyboard(const Controller& controller)
     return;
   }
 
-  if (controller.pressed(Controller::ESCAPE)) {
+  if (controller.pressed(Control::ESCAPE)) {
     esc_press();
     return;
   }
 
-  if (controller.hold(Controller::LEFT)) {
+  if (controller.hold(Control::LEFT)) {
     scroll({-32.0f, 0.0f});
   }
 
-  if (controller.hold(Controller::RIGHT)) {
+  if (controller.hold(Control::RIGHT)) {
     scroll({32.0f, 0.0f});
   }
 
-  if (controller.hold(Controller::UP)) {
+  if (controller.hold(Control::UP)) {
     scroll({0.0f, -32.0f});
   }
 
-  if (controller.hold(Controller::DOWN)) {
+  if (controller.hold(Control::DOWN)) {
     scroll({0.0f, 32.0f});
   }
 }
@@ -429,11 +423,15 @@ Editor::set_level(std::unique_ptr<Level> level, bool reset)
   }
 
   load_sector(sector_name);
-  m_sector->activate(sector_name);
-  m_sector->get_camera().set_mode(Camera::Mode::MANUAL);
 
-  if (!reset) {
-    m_sector->get_camera().set_translation(translation);
+  if (m_sector != nullptr)
+  {
+    m_sector->activate(sector_name);
+    m_sector->get_camera().set_mode(Camera::Mode::MANUAL);
+
+    if (!reset) {
+      m_sector->get_camera().set_translation(translation);
+    }
   }
 
   m_layers_widget->refresh_sector_text();
@@ -545,15 +543,14 @@ Editor::setup()
   }
   m_toolbox_widget->setup();
   m_layers_widget->setup();
-  m_savegame.reset(new Savegame("levels/misc"));
-  m_savegame->load();
+  m_savegame = Savegame::from_file("levels/misc");
 
   // Reactivate the editor after level test
   if (m_leveltested) {
     if (!m_test_levelfile.empty())
     {
       // Try to remove the test level using the PhysFS file system
-      if (PhysFSFileSystem::remove(m_test_levelfile) != 0)
+      if (physfsutil::remove(m_test_levelfile) != 0)
       {
         // This file is not inside any PhysFS mounts,
         // try to remove this using normal file system
@@ -588,19 +585,30 @@ Editor::event(const SDL_Event& ev)
   if (!m_enabled) return;
 
   try
-  {
-    // undo/redo key combo
-    if (ev.type == SDL_KEYDOWN &&
+  {	
+	if (ev.type == SDL_KEYDOWN &&
+        ev.key.keysym.sym == SDLK_t &&
+        ev.key.keysym.mod & KMOD_CTRL) {
+		test_level();
+		}
+		
+	if (ev.type == SDL_KEYDOWN &&
+        ev.key.keysym.sym == SDLK_s &&
+        ev.key.keysym.mod & KMOD_CTRL) {
+		save_level();
+		}
+		
+	if (ev.type == SDL_KEYDOWN &&
         ev.key.keysym.sym == SDLK_z &&
-        ev.key.keysym.mod & KMOD_CTRL)
-    {
-      if (ev.key.keysym.mod & KMOD_SHIFT) {
-        redo();
-      } else {
-        undo();
-      }
-      return;
-    }
+        ev.key.keysym.mod & KMOD_CTRL) {
+		undo();
+		}
+		
+	if (ev.type == SDL_KEYDOWN &&
+        ev.key.keysym.sym == SDLK_y &&
+        ev.key.keysym.mod & KMOD_CTRL) {
+		redo();
+		}
 
     if (ev.type == SDL_KEYDOWN && ev.key.keysym.sym == SDLK_F6) {
       Compositor::s_render_lighting = !Compositor::s_render_lighting;
@@ -625,9 +633,18 @@ Editor::event(const SDL_Event& ev)
          ev.type == SDL_MOUSEBUTTONUP))
     {
       if (!m_ignore_sector_change) {
-        m_undo_manager->try_snapshot(*m_level);
+        if (m_level) {
+          m_undo_manager->try_snapshot(*m_level);
+        }
       }
     }
+
+    // scroll with mouse wheel
+    if (ev.type == SDL_MOUSEWHEEL) {
+      float scroll_x = static_cast<float>(ev.wheel.x * -32);
+      float scroll_y = static_cast<float>(ev.wheel.y * -32);
+      scroll({scroll_x, scroll_y});
+    }
   }
   catch(const std::exception& err)
   {
