@@ -252,17 +252,25 @@ Camera::reset(const Vector& tuxpos)
 }
 
 void
-Camera::shake(float time, float x, float y)
+Camera::shake(float duration, float x, float y)
 {
-  m_shaketimer.start(time);
+  m_shaketimer.start(duration);
   m_shakedepth_x = x;
   m_shakedepth_y = y;
-  m_shakespeed = math::PI_2 / time;
+  m_shakespeed = math::PI_2 / duration;
 }
 
 void
 Camera::scroll_to(const Vector& goal, float scrolltime)
 {
+  if(scrolltime == 0.0f)
+  {
+    m_translation.x = goal.x;
+    m_translation.y = goal.y;
+    m_mode = Mode::MANUAL;
+    return;
+  }
+
   m_scroll_from = m_translation;
   m_scroll_goal = goal;
   keep_in_bounds(m_scroll_goal);
@@ -365,9 +373,9 @@ Camera::update_scroll_normal(float dt_sec)
     m_cached_translation.y = player_pos.y - static_cast<float>(m_screen_size.height) * config_.target_y;
   }
   if (ymode == 2) {
-    // target_y is the high we target our scrolling at. This is not always the
-    // high of the player, but if he is jumping upwards we should use the
-    // position where he last touched the ground. (this probably needs
+    // target_y is the height we target our scrolling at. This is not always the
+    // height of the player: while jumping upwards, we should use the
+    // position where they last touched the ground. (this probably needs
     // exceptions for trampolines and similar things in the future)
     float target_y;
     if (player.m_fall_mode == Player::JUMPING)
@@ -516,7 +524,7 @@ Camera::update_scroll_normal(float dt_sec)
 
       m_changetime = -1;
     } else if (m_lookahead_mode != walkDirection) {
-      /* player changed direction while camera was scrolling...
+      /* Tux changed direction while camera was scrolling...
        * he has to do this for a certain time to add robustness against
        * sudden changes */
       if (m_changetime < 0) {
