@@ -19,19 +19,28 @@
 #include "supertux/sector.hpp"
 #include "video/drawing_context.hpp"
 
-SpecialRiser::SpecialRiser(const Vector& pos, std::unique_ptr<MovingObject> child_) :
-  offset(0),
-  child(std::move(child_))
+SpecialRiser::SpecialRiser(const Vector& pos, std::unique_ptr<MovingObject> child, bool is_solid) :
+  m_start_pos(),
+  m_offset(0),
+  m_child(std::move(child))
 {
-  child->set_pos(pos - Vector(0, 32));
+  m_start_pos = pos;
+  m_child->set_pos(pos - Vector(0,32));
+  set_pos(m_start_pos);
+  m_col.m_bbox.set_size(m_child->get_bbox().get_width(), 32);
+  if (is_solid)
+    set_group(COLGROUP_STATIC);
+  else
+    set_group(COLGROUP_DISABLED);
 }
 
 void
 SpecialRiser::update(float dt_sec)
 {
-  offset += 50 * dt_sec;
-  if (offset > 32) {
-    Sector::get().add_object(std::move(child));
+  m_offset += 50 * dt_sec;
+  set_pos(m_start_pos - Vector(0, m_offset));
+  if (m_offset > 32) {
+    Sector::get().add_object(std::move(m_child));
     remove_me();
   }
 }
@@ -41,8 +50,8 @@ SpecialRiser::draw(DrawingContext& context)
 {
   context.push_transform();
   context.set_translation(
-    context.get_translation() + Vector(0, -32 + offset));
-  child->draw(context);
+    context.get_translation() + Vector(0, -32 + m_offset));
+  m_child->draw(context);
   context.pop_transform();
 }
 
