@@ -17,7 +17,6 @@
 #include "object/tilemap.hpp"
 
 #include <tuple>
-#include <cmath>
 
 #include "editor/editor.hpp"
 #include "supertux/debug.hpp"
@@ -113,7 +112,7 @@ TileMap::TileMap(const TileSet *tileset_, const ReaderMapping& reader) :
     }
   }
 
-  m_z_pos = reader_get_layer (reader, /* default = */ 0);
+  m_z_pos = reader_get_layer(reader, 0);
 
   if (!Editor::is_active()) {
     if (m_real_solid && ((m_speed_x != 1) || (m_speed_y != 1))) {
@@ -349,15 +348,11 @@ TileMap::draw(DrawingContext& context)
     context.set_alpha(m_current_alpha/2);
   }
 
-  // Force the translation to be an integer so that the tiles appear sharper.
-  // For consistency (i.e., to avoid 1-pixel gaps), this needs to be done even
-  // for solid tilemaps that are guaranteed to have speed 1.
-  // FIXME Force integer translation for all graphics, not just tilemaps.
-  float trans_x = roundf(context.get_translation().x);
-  float trans_y = roundf(context.get_translation().y);
-  bool normal_speed = m_editor_active && Editor::is_active();
-  context.set_translation(Vector(std::truncf(trans_x * (normal_speed ? 1.0f : m_speed_x)),
-                                 std::truncf(trans_y * (normal_speed ? 1.0f : m_speed_y))));
+  const float trans_x = context.get_translation().x;
+  const float trans_y = context.get_translation().y;
+  const bool normal_speed = m_editor_active && Editor::is_active();
+  context.set_translation(Vector(trans_x * (normal_speed ? 1.0f : m_speed_x),
+                                 trans_y * (normal_speed ? 1.0f : m_speed_y)));
 
   Rectf draw_rect = context.get_cliprect();
   Rect t_draw_rect = get_tiles_overlapping(draw_rect);
@@ -383,7 +378,7 @@ TileMap::draw(DrawingContext& context)
         tile.draw_debug(context.color(), pos, LAYER_FOREGROUND1);
       }
 
-      const SurfacePtr& surface = Editor::current() ? tile.get_current_editor_surface() : tile.get_current_surface();
+      const SurfacePtr& surface = Editor::is_active() ? tile.get_current_editor_surface() : tile.get_current_surface();
       if (surface) {
         std::get<0>(batches[surface]).emplace_back(surface->get_region());
         std::get<1>(batches[surface]).emplace_back(pos,
