@@ -20,13 +20,13 @@
 #include <squirrel.h>
 #include <vector>
 
-/**
- * Keeps a list of SquirrelThreads that wait for a wakeup event
- */
+class SquirrelVM;
+
+/** Keeps a list of SquirrelThreads that wait for a wakeup event */
 class SquirrelThreadQueue final
 {
 public:
-  SquirrelThreadQueue(HSQUIRRELVM vm);
+  SquirrelThreadQueue(SquirrelVM& vm);
 
   /** adds a thread (actually a weakref to the thread) */
   void add(HSQUIRRELVM vm);
@@ -35,7 +35,7 @@ public:
   void wakeup();
 
 private:
-  HSQUIRRELVM m_vm;
+  SquirrelVM& m_vm;
   std::vector<HSQOBJECT> m_threads;
 
 private:
