@@ -24,6 +24,8 @@ Debug g_debug;
 Debug::Debug() :
   show_collision_rects(false),
   show_worldmap_path(false),
+  show_controller(false),
+  draw_redundant_frames(false),
   m_use_bitmap_fonts(false),
   m_game_speed_multiplier(1.0f)
 {
