@@ -35,6 +35,12 @@ public:
   /** Draw the path on the worldmap, including invisible paths */
   bool show_worldmap_path;
 
+  bool show_controller;
+
+  // Draw frames even when visually nothing changes; this can be used to
+  // vaguely measure the impact of code changes which should increase the FPS
+  bool draw_redundant_frames;
+
 private:
   /** Use old bitmap fonts instead of TTF */
   bool m_use_bitmap_fonts;
