@@ -159,8 +159,8 @@ public:
   PhysfsSubsystem(const char* argv0,
                   boost::optional<std::string> forced_datadir,
                   boost::optional<std::string> forced_userdir) :
-    m_forced_datadir(forced_datadir),
-    m_forced_userdir(forced_userdir)
+    m_forced_datadir(std::move(forced_datadir)),
+    m_forced_userdir(std::move(forced_userdir))
   {
     if (!PHYSFS_init(argv0))
     {
@@ -233,10 +233,14 @@ public:
 		userdir = PHYSFS_getPrefDir("SuperTux","supertux2");
     }
 	//Kept for backwards-compatability only, hence the silence
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#ifdef __GNUC__
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 	std::string physfs_userdir = PHYSFS_getUserDir();
-#pragma GCC diagnostic pop
+#ifdef __GNUC__
+  #pragma GCC diagnostic pop
+#endif
 
 #ifdef _WIN32
 	std::string olduserdir = FileSystem::join(physfs_userdir, PACKAGE_NAME);
@@ -349,7 +353,7 @@ public:
 void
 Main::init_video()
 {
-  VideoSystem::current()->set_title(PACKAGE_NAME " " PACKAGE_VERSION);
+  VideoSystem::current()->set_title("SuperTux " PACKAGE_VERSION);
 
   const char* icon_fname = "images/engine/icons/supertux-256x256.png";
 
